<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\Collection\Filters;

defined('_JEXEC') or die;


use JDatabaseQuery;
use RegularLabs\Library\ArrayHelper as RL_Array;
use RegularLabs\Library\DB as RL_DB;
use RegularLabs\Plugin\System\ArticlesAnywhere\Collection\DB;

class Tags extends Filter
{
    var $names;

    public function setConditionsWhenEmpty(JDatabaseQuery $query)
    {
        $tag_ids       = DB::getResults($this->getTagsIdsQuery(['*']), 'loadColumn');
        $ids_with_tags = $this->getItemIdsByTagIds($tag_ids);

        $ids_with_tags[0] = '!' . $ids_with_tags[0];

        $query->where($this->db->quoteName('items.id') . RL_DB::in($ids_with_tags));
    }

    public function setFilter(JDatabaseQuery $query, $filters = [])
    {
        $this->names = $filters;

        $tag_ids = $this->getTagIds();

        $include = $this->getItemIdsByTagIds($tag_ids->include, $tag_ids->match_all);
        $exclude = $this->getItemIdsByTagIds($tag_ids->exclude, $tag_ids->match_all);

        if (empty($include) && empty($exclude))
        {
            $query->where('0');

            return false;
        }

        if ( ! empty($include))
        {
            $query->where($this->db->quoteName('items.id') . RL_DB::in($include));
        }

        if ( ! empty($exclude))
        {
            $exclude[0] = '!' . $exclude[0];
            $query->where($this->db->quoteName('items.id') . RL_DB::in($exclude));
        }

        return true;
    }

    protected function getItemIdsByTagIds($ids, $match_all = false)
    {
        if (empty($ids))
        {
            return [];
        }

        $include_children = $this->config->getFiltersIncludeChildren('tags');

        if ($include_children)
        {
            $max_depth = is_int($include_children) ? $include_children : 20;
            $depth     = 0;

            $child_ids = $ids;

            while ($depth++ < $max_depth && $child_ids = $this->getChildIds($child_ids))
            {
                $ids = array_merge($ids, $child_ids);
            }
        }

        return $match_all
            ? $this->getItemIdsByTagIdsMatchAll($ids)
            : $this->getItemIdsByTagIdsMatchAny($ids);
    }

    protected function getItemIdsByTagIdsMatchAll($ids)
    {
        $first_id = array_shift($ids);
        $item_ids = $this->getItemIdsByTagIds($first_id);

        foreach ($ids as $id)
        {
            $item_ids = array_intersect(
                $item_ids,
                $this->getItemIdsByTagIds($id)
            );

            if (empty($item_ids))
            {
                return [];
            }
        }

        return $item_ids;
    }

    protected function getItemIdsByTagIdsMatchAny($ids)
    {
        $query = $this->db->getQuery(true)
            ->select($this->db->quoteName('content_item_id'))
            ->from($this->db->quoteName('#__contentitem_tag_map'))
            ->where($this->db->quoteName('type_alias') . ' = ' . $this->db->quote('com_content.article'))
            ->where($this->db->quoteName('tag_id') . RL_DB::in($ids))
            ->group($this->db->quoteName('content_item_id'));

        return DB::getResults($query) ?: [];
    }

    protected function getTagIds()
    {
        $includes  = [];
        $excludes  = [];
        $match_all = false;

        foreach ($this->names as $name)
        {
            if ($name == '!' || $name == '!NOT!')
            {
                $name = '+';
            }

            $operator = RL_DB::getOperatorFromValue($name);
            $name     = RL_DB::removeOperator($name);

            $name = [$name];

            if (strpos($name[0], '+') !== false)
            {
                $name      = RL_Array::toArray($name[0], '+');
                $match_all = true;
            }

            if ($operator == '!=')
            {
                $excludes = array_merge($excludes, $name);
                continue;
            }

            $includes = array_merge($includes, $name);
        }

        $include = ! empty($includes) ? DB::getResults($this->getTagsIdsQuery($includes), 'loadColumn') : [];
        $exclude = ! empty($excludes) ? DB::getResults($this->getTagsIdsQuery($excludes), 'loadColumn') : [];

        return (object) compact('include', 'exclude', 'match_all');
    }

    protected function getTagsIdsQuery($names)
    {
        $query = $this->db->getQuery(true)
            ->select($this->db->quoteName('id'))
            ->from($this->config->getTableTags('tags'));

        $this->setFiltersFromNames($query, 'tags', $names, 'also');
        $this->setIgnores($query, 'tags', 'tags');

        return $query;
    }

    private function getChildIds($ids = [])
    {
        $query = $this->getChildIdsQuery($ids);

        return DB::getResults($query) ?: [];
    }

    private function getChildIdsQuery($parent_ids = [])
    {
        $query = $this->db->getQuery(true)
            ->select($this->db->quoteName('id'))
            ->from($this->config->getTableTags('tags'))
            ->where($this->db->quoteName('parent_id') . RL_DB::in($parent_ids));

        return $query;
    }
}
