<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\Output\Data;

defined('_JEXEC') or die;


use FieldsHelper;
use JEventDispatcher;
use JLoader;
use Joomla\CMS\Language\Text as JText;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Plugin\PluginHelper as JPluginHelper;
use Joomla\Registry\Registry;
use RegularLabs\Library\ArrayHelper as RL_Array;
use RegularLabs\Plugin\System\ArticlesAnywhere\Collection\Fields\CustomFields as Fields;
use RegularLabs\Plugin\System\ArticlesAnywhere\Collection\Item;
use RegularLabs\Plugin\System\ArticlesAnywhere\Config;
use RegularLabs\Plugin\System\ArticlesAnywhere\Factory;
use RegularLabs\Plugin\System\ArticlesAnywhere\Helpers\ValueHelper;
use RegularLabs\Plugin\System\ArticlesAnywhere\Output\Values;

class CustomFields extends Data
{
    private $fields = [];
    /* @var Images $images */
    private $images;
    /* @var Text $text */
    private $text;

    public function __construct(Config $config, Item $item, Values $values)
    {
        parent::__construct($config, $item, $values);

        JLoader::register('FieldsHelper', JPATH_ADMINISTRATOR . '/components/com_fields/helpers/fields.php');

        $this->text   = Factory::getOutput('Text', $config, $item, $values);
        $this->images = Factory::getOutput('Images', $config, $item, $values);
        $this->fields = FieldsHelper::getFields('com_content.article', $this->item->get());
    }

    public function get($key, $attributes)
    {
        $fields = (new Fields($this->config))->getFieldByKey($key, $this->item);

        if (empty($fields))
        {
            return null;
        }

        if ( ! is_array($fields))
        {
            return '';
        }

        $field = $this->getField($fields, $attributes);

        if (is_null($field))
        {
            return '';
        }

        if (isset($field->params->showlabel) && $field->params->showlabel === 'only')
        {
            return $field->label;
        }

        if ($fields[0]->type == 'repeatable')
        {
            $this->prepareRepeatableField($fields, $field->params);
        }

        // Should we combine attributes with the $fieldparams ?
        if (empty($field->rawvalue) || empty($field->value))
        {
            return $this->getOutput($field, $attributes);
        }

        if ( ! $this->values->isDateValue($key, RL_Array::implode($field->rawvalue)) &&
            ! $this->values->isDateValue($key, $field->value))
        {
            return $this->getOutput($field, $attributes);
        }

        if ( ! isset($attributes->showtime) && isset($field->fieldparams->showtime))
        {
            $attributes->showtime = $field->fieldparams->showtime;
        }

        $attributes->is_custom_field = true;

        return $this->getOutput($field, $attributes);
    }

    private function convertValuesToDate(&$values, $attributes)
    {
        if (empty($attributes->format))
        {
            return;
        }

        foreach ($values as &$value)
        {
            if ( ! ValueHelper::isDateValue($value))
            {
                continue;
            }

            // Convert string if it is a date
            $value = ValueHelper::dateToString($value, $attributes);
        }
    }

    private function getField($fields, $attributes)
    {
        $return_field = null;
        $values       = [];

        foreach ($fields as &$field)
        {
            if (is_null($return_field))
            {
                $return_field = $field;
            }

            $value = $this->getItemValue($field, $attributes);

            if ($value === '')
            {
                continue;
            }

            $rawvalue = ($field->rawvalue ?? null) ?: $field->value;
            $rawvalue = RL_Array::implode($rawvalue);

            $values[$rawvalue] = $value;
        }

        if (is_null($return_field))
        {
            return null;
        }

        self::convertValuesToDate($values, $attributes);

        $field = clone $return_field;

        $attributes->separator ??= $attributes->delimiter ?? ', ';
        unset($attributes->delimiter);

        $field->label       = JText::_($field->label);
        $field->params      = isset($field->params) ? (is_object($field->params) ? $field->params : json_decode($field->params)) : json_decode('{}');
        $field->fieldparams = isset($field->fieldparams) ? (is_object($field->fieldparams) ? $field->fieldparams : json_decode($field->fieldparams)) : json_decode('{}');
        $field->value       = $values;
        $field->rawvalue    = array_keys($values);

        // Switch showlabel to off if not set and no layout is set
        if (empty($attributes->showlabel)
            && (empty($attributes->layout) || ($attributes->layout === false || $attributes->layout === 'false'))
        )
        {
            $attributes->showlabel = false;
        }

        if ( ! isset($attributes->layout)
            || $attributes->layout === true
            || $attributes->layout === 'true'
        )
        {
            $attributes->layout = ($field->params->layout ?? null) ?: 'default';
        }

        if ($attributes->layout === false
            || $attributes->layout === 'false'
        )
        {
            $attributes->layout = '';
        }

        $field->params->layout = $attributes->layout;
        unset($attributes->layout);

        $field->params = (object) array_merge((array) $field->params, (array) $attributes);

        if (empty($field->params->value_layout))
        {
            $field->value = implode($field->params->separator, $field->value);

            return $field;
        }

        $field->value = $this->getValueLayout($field->params->value_layout, $values, $field->params);

        return $field;
    }

    private function getFieldObject($field_id)
    {
        foreach ($this->fields as $field)
        {
            if ($field->id == $field_id)
            {
                return $field;
            }
        }

        return false;
    }

    private function getItemValue($field, $attributes)
    {
        if ( ! isset($field->value) || $field->value == '')
        {
            return '';
        }

        if (in_array($field->type, ['text', 'textarea', 'editor']))
        {
            return $this->text->process($field->value, $attributes);
        }

        if (empty($field->fieldparams))
        {
            $field->fieldparams = '{}';
        }

        $params = is_string($field->fieldparams) ? json_decode($field->fieldparams) : $field->fieldparams;

        if ( ! empty($params->options))
        {
            return $this->getItemValueByOptions($field, $params->options, $attributes);
        }

        return $this->getValueFromField($field, $attributes);
    }

    private function getItemValueByOptions($field, $options, $attributes)
    {
        foreach ($options as $option)
        {
            if ($field->value != $option->value)
            {
                continue;
            }

            if (isset($attributes->output)
                && in_array($attributes->output, ['value', 'values', 'raw']))
            {

                return $option->value;
            }

            return JText::_($option->name);
        }

        return $field->value;
    }

    private function getItemValueMedia($field, $attributes)
    {
        $image = $this->images->getImageByUrl($field->value, $attributes);

        if (isset($attributes->output)
            && in_array($attributes->output, ['value', 'values', 'raw']))
        {
            return $image['url'];
        }

        if ( ! empty($attributes->clean))
        {
            return $image['url'];
        }

        $params = is_string($field->fieldparams) ? json_decode($field->fieldparams) : $field->fieldparams;

        $attributes->src   = $image['url'];
        $attributes->class = isset($attributes->class)
            ? $attributes->class
            : ($params->image_class ?? '');
        Images::setAltAndTitle('custom_fields', $attributes, $field);

        $image_attributes = clone $attributes;

        unset($image_attributes->clean);
        unset($image_attributes->layout);
        unset($image_attributes->value_layout);
        unset($image_attributes->showlabel);

        return $this->images->getImageHtml($image_attributes);
    }

    private function getLayoutPathDotted($path, $prefix = 'field')
    {
        $path = str_replace('.php', '', $path);
        $path = str_replace('/', '.', trim($path, '/'));

        if (strpos($path, '.') === false)
        {
            $path = $prefix . '.' . $path;
        }

        return $path;
    }

    private function getOutput($field)
    {
        // Don't output anything if value is empty
        if ( ! isset($field->value) || $field->value == '')
        {
            return '';
        }

        $result_from_layout = $this->getOutputViaLayout($field);

        if ($result_from_layout !== false)
        {
            return $result_from_layout;
        }

        if ( ! $field->params->showlabel)
        {
            return $field->value;
        }

        $format = $field->params->format ?? '%s: %s';

        return sprintf($format, $field->label, $field->value);
    }

    /**
     * @param $field
     * @param $params
     *
     * @return string
     */
    private function getOutputViaLayout($field)
    {
        if (empty($field->params->layout))
        {
            return false;
        }

        if (isset($field->params->output)
            && in_array($field->params->output, ['value', 'values', 'raw']))
        {
            return false;
        }

        $field->params->layout = ($field->params->layout === true || $field->params->layout == 'true' || $field->params->layout == 'default')
            ? 'field.render'
            : $this->getLayoutPathDotted($field->params->layout, 'field');

        $layout_field         = clone $field;
        $layout_field->params = new Registry($layout_field->params);

        $displayData = [
            'field'      => $layout_field,
            'attributes' => $field->params,
            'item'       => $this->item->get(),
        ];

        $layout = new FileLayout($field->params->layout, JPATH_SITE, ['component' => 'com_fields', 'client' => 0]);

        $output = $layout->render($displayData);

        foreach ($layout->getDebugMessages() as $message)
        {
            if (strpos($message, 'Unable to find layout') === false)
            {
                continue;
            }

            $field->params->layout = 'field.render';
            $layout                = new FileLayout($field->params->layout, JPATH_SITE, ['component' => 'com_fields', 'client' => 0]);

            $output = $layout->render($displayData);
            break;
        }

        return trim($output);
    }

    private function getValueFromField($field, $attributes)
    {
        $value = $field->value;

        // Do not use the raw output if the $attributes value is set to 'output' or 'render'
        if (isset($attributes->value)
            && in_array($attributes->value, ['output', 'render']))
        {
            unset($attributes->output);
            unset($attributes->value);
        }

        if (in_array($field->type, ['media']))
        {
            return $this->getItemValueMedia($field, $attributes);
        }

        if (isset($attributes->output)
            && in_array($attributes->output, ['value', 'values', 'raw']))
        {
            return $value;
        }

        $field_object = $this->getFieldObject($field->id);

        if ( ! $field_object)
        {
            return $value;
        }

        $field_object->value = $field->value;

        if (isset($attributes->format) && ValueHelper::isDateValue($value))
        {
            // timezone and formatting will be applied at a later stage
            return $value;
        }

        $field_type = $field->type == 'dpevent' ? 'dpcalendar' : $field->type;

        if ( ! JPluginHelper::importPlugin('fields', $field_type))
        {
            return $value;
        }

        $plugin_class = 'PlgFields' . ucfirst($field_type);

        $dispatcher = JEventDispatcher::getInstance();
        $params     = (array) JPluginHelper::getPlugin('fields', $field_type);
        $plugin     = new $plugin_class($dispatcher, $params);

        if (method_exists($plugin, 'onCustomFieldsBeforePrepareField'))
        {
            $plugin->onCustomFieldsBeforePrepareField('com_content.article', $this->item->get(), $field_object);
        }

        $value = $plugin->onCustomFieldsPrepareField('com_content.article', $this->item->get(), $field_object);

        if (is_array($value))
        {
            $value = implode(' ', $value);
        }

        if (method_exists($plugin, 'onCustomFieldsAfterPrepareField'))
        {
            $plugin->onCustomFieldsAfterPrepareField('com_content.article', null, $field_object, $value);
        }

        return $value;
    }

    private function getValueLayout($value_layout, $values, $attributes)
    {
        $value_layout = $this->getLayoutPathDotted($value_layout);
        $displayData  = [
            'values'     => $values,
            'rawvalues'  => array_keys($values),
            'attributes' => $attributes,
        ];

        $value_layout = new FileLayout($value_layout, JPATH_SITE, ['component' => 'com_fields', 'client' => 0]);

        $output = $value_layout->render($displayData);

        foreach ($value_layout->getDebugMessages() as $message)
        {
            if (strpos($message, 'Unable to find layout') !== false)
            {
                return null;
            }
        }

        return $output;
    }

    private function prepareRepeatableField($fields, $attributes)
    {
        $keys = ! empty($attributes->fields) ? RL_Array::toArray($attributes->fields) : [];

        foreach ($fields as &$field)
        {
            if (empty($field->value))
            {
                continue;
            }

            $value = json_decode($field->value);

            if (empty($value))
            {
                continue;
            }

            $fieldparams = is_string($field->fieldparams) ? json_decode($field->fieldparams) : $field->fieldparams;

            $types = [];

            foreach ($fieldparams->fields as $key => $fieldparam)
            {
                if ( ! empty($keys) && ! in_array($fieldparam->fieldname, $keys))
                {
                    unset($fieldparams->fields->{$key});
                    continue;
                }

                $types[$fieldparam->fieldname] = $fieldparam->fieldtype;
            }

            foreach ($value as $key => &$row)
            {
                if ( ! empty($keys))
                {
                    // Filter and order keys in order of given
                    $row = array_merge(
                        array_flip($keys),
                        array_intersect_key(
                            (array) $row,
                            array_flip($keys)
                        )
                    );
                }

                foreach ($row as $row_key => &$val)
                {
                    if ($types[$row_key] != 'media')
                    {
                        continue;
                    }

                    // TODO: This doesn't work correctly yet
                    // Figure out what data to pass on
                    return $this->getItemValueMedia($field, $attributes);
                }
            }

            $field->value       = json_encode($value);
            $field->fieldparams = json_encode($fieldparams);
        }
    }
}
