<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\DataGroups\Helpers;

defined('_JEXEC') or die;

use Joomla\CMS\Factory as JFactory;
use Joomla\Component\Fields\Administrator\Helper\FieldsHelper as JFieldsHelper;
use Joomla\Utilities\ArrayHelper;
use RegularLabs\Library\Cache as RL_Cache;
use RegularLabs\Library\DB as RL_DB;
use RegularLabs\Library\ObjectHelper as RL_Object;
use RegularLabs\Library\StringHelper as RL_String;

class Fields
{
    public static function getField($id, $article, $prepare_value = true)
    {
        if ( ! $article->id)
        {
            return null;
        }

        $fields = self::getFields($article, $prepare_value);

        if (isset($fields[$id]))
        {
            return RL_Object::clone($fields[$id]);
        }

        foreach ($fields as $field)
        {
            if (
                (is_numeric($id) && (int) $field->id === (int) $id)
                || (RL_String::toDashCase($field->title) === $id)
                || (RL_String::toDashCase($field->label) === $id)
            )
            {
                return RL_Object::clone($field);
            }
        }

        return null;
    }

    public static function getFieldGroups()
    {
        $user       = JFactory::getApplication()->getIdentity() ?: JFactory::getUser();
        $viewlevels = ArrayHelper::toInteger($user->getAuthorisedViewLevels());

        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select(['a.*']);
        $query->from($db->quoteName('#__fields_groups', 'a'));
        $query->where('a.context = ' . $db->quote('com_content.article'));
        $query->whereIn('a.access', $viewlevels);

        $db->setQuery($query);

        return $db->loadObjectList('id');
    }

    public static function getFieldType($id)
    {
        $fields = self::getFieldTypes();

        foreach ($fields as $field)
        {
            if (
                (is_numeric($id) && (int) $field->id === (int) $id)
                || (RL_String::toDashCase($field->name) === $id)
            )
            {
                return $field;
            }
        }

        return null;
    }

    public static function getFieldTypes()
    {
        $cache = new RL_Cache;

        if ($cache->exists())
        {
            return $cache->get();
        }

        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__fields', 'a'))
            ->whereIn('a.state', [0, 1])
            ->where(RL_DB::is('a.context', 'com_content.article'));

        $db->setQuery($query);

        $fields = $db->loadObjectList();

        foreach ($fields as &$field)
        {
            $field->params      = json_decode($field->params ?: '{}');
            $field->fieldparams = json_decode($field->fieldparams ?: '{}');
        }

        return $cache->set($fields);
    }

    public static function getFields($article, $prepare_value = true)
    {
        $cache = new RL_Cache([__METHOD__, $article->id, $prepare_value]);

        if ($cache->exists())
        {
            return $cache->get();
        }

        $fields = JFieldsHelper::getFields('com_content.article', $article, $prepare_value);

        if (empty($fields))
        {
            return $cache->set([]);
        }

        $article_fields = [];

        foreach ($fields as $field)
        {
            // flatten params to main field object
            $field = RL_Object::merge($field, $field->params->toObject());

            if ($field->fieldparams->get('multiple') === 1)
            {
                $field->value    = is_array($field->value) ? $field->value : [$field->value];
                $field->rawvalue = is_array($field->rawvalue) ? $field->rawvalue : [$field->rawvalue];
            }

            $article_fields[RL_String::toDashCase($field->name)] = $field;
        }

        return $cache->set($article_fields);
    }

    public static function getFieldsByGroup($group_id, $article, $prepare_value = true)
    {
        if ( ! is_numeric($group_id))
        {
            $group_id = self::getGroupIdByTitle($group_id);
        }

        if ( ! $group_id)
        {
            return [];
        }

        $fields = self::getFields($article, $prepare_value);

        $render_fields = [];

        foreach ($fields as $field)
        {
            if ($field->group_id === $group_id)
            {
                $render_fields[] = $field;
            }
        }

        return $render_fields;
    }

    public static function getGroup($group_id)
    {
        if ( ! is_numeric($group_id))
        {
            $group_id = self::getGroupIdByTitle($group_id);
        }

        $groups = self::getFieldGroups();

        return $groups[$group_id] ?? null;
    }

    public static function getGroupIdByTitle($group_title)
    {
        $group_title = RL_String::toDashCase($group_title);
        $groups      = self::getFieldGroups();

        foreach ($groups as $group)
        {
            if (RL_String::toDashCase($group->title) === $group_title)
            {
                return $group->id;
            }
        }

        return 0;
    }
}
