<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\DataGroups;

defined('_JEXEC') or die;

use Joomla\CMS\Factory as JFactory;
use Joomla\CMS\Helper\TagsHelper as JTagsHelper;
use RegularLabs\Library\ArrayHelper as RL_Array;
use RegularLabs\Plugin\System\ArticlesAnywhere\DataGroups\Helpers\Tags as TagsHelper;
use RegularLabs\Plugin\System\ArticlesAnywhere\Helpers\Data;
use RegularLabs\Plugin\System\ArticlesAnywhere\Helpers\DB;

class Tags extends DataGroup
{
    protected static $default_data_key = 'layout';
    protected static $main_table       = '';
    protected static $prefix           = 'tags';

    /**
     * @return mixed
     */
    public function getForeachData()
    {
        $tags = $this->getTagsItems();

        foreach ($tags as $tag)
        {
            $tag->text = $tag->title;
        }

        return $tags;
    }

    public function getOutputAliases()
    {
        $tags    = $this->getTagsItems();
        $aliases = TagsHelper::getTagAliases($tags);

        return RL_Array::implode(
            $aliases,
            $this->getAttribute('separator', ','),
            $this->getAttribute('last-separator')
        );
    }

    /**
     * @return mixed
     */
    public function getOutputRaw()
    {
        $tags = $this->getTagsItems();

        return TagsHelper::getTagTitles($tags);
    }

    public function getOutputTitles()
    {
        $tags   = $this->getTagsItems();
        $titles = TagsHelper::getTagTitles($tags);

        return RL_Array::implode(
            $titles,
            $this->getAttribute('separator', ','),
            $this->getAttribute('last-separator')
        );
    }

    public function getRequiredQueryKeys()
    {
        return ['article.id'];
    }

    public function getSubQueryTypesForWheres($values, $glue = 'OR')
    {
        if (empty($values))
        {
            return [[], []];
        }

        if ($glue === 'AND')
        {
            $includes = [];
            $excludes = [];

            foreach ($values as $value)
            {
                [$value_includes, $value_excludes] = $this->getSubQueryTypesForWheres([$value], 'OR');

                $includes = array_merge($includes, $value_includes);
                $excludes = array_merge($excludes, $value_excludes);
            }

            $includes = RL_Array::clean($includes);
            $excludes = RL_Array::clean($excludes);

            return [$includes, $excludes];
        }

        [$includes, $excludes] = DB::getIncludesExcludes($values);

        $includes = [self::getSubQueriesForWheres($includes, $glue)];
        $excludes = [self::getSubQueriesForWheres($excludes, $glue)];

        $includes = RL_Array::clean($includes);
        $excludes = RL_Array::clean($excludes);

        return [$includes, $excludes];
    }

    /**
     * @return array
     */
    public function getTagsItems()
    {
        $tags_helper = new JTagsHelper;
        $tags_helper->getItemTags('com_content.article', $this->get('article.id'));

        return RL_Array::toArray($tags_helper->itemTags ?? []);
    }

    /**
     * @return mixed
     */
    public function getValue()
    {
        $tags = $this->getTagsItems();

        if (empty($tags))
        {
            return '';
        }

        $layout    = $this->getAttribute('layout', true);
        $use_links = $this->getAttribute('links', true);
        $separator = $this->getAttribute('separator', ', ');

        if ( ! $use_links || $separator)
        {
            $layout = false;
        }

        if ((JFactory::getApplication()->input->get('option', '') === 'com_finder'
            && JFactory::getApplication()->input->get('format', '') === 'json')
        )
        {
            // Force text output without links for finder indexing, as the Tags RouteHelper causes errors
            $layout    = false;
            $use_links = false;
        }

        if ( ! $layout)
        {
            return TagsHelper::getOutput($tags, $separator, $this->getAttribute('last-separator'), $use_links);
        }

        return TagsHelper::getOutputViaLayout($tags, $this->getAttribute('layout'));
    }

    /**
     * @param array|object|string $values
     * @param bool                $exclude
     *
     * @return string
     */
    public function getWhere($values, $glue = 'OR')
    {
        if (empty($values))
        {
            return '';
        }

        $values = Data::valuesToSimpleArray($values);

        [$includes, $excludes] = self::getSubQueryTypesForWheres($values, $glue);

        $wheres = [];
        foreach ($includes as $sub_query)
        {
            $wheres[] = DB::quoteName('article.id') . ' IN (' . (string) $sub_query . ')';
        }
        foreach ($excludes as $sub_query)
        {
            $wheres[] = DB::quoteName('article.id') . ' NOT IN (' . (string) $sub_query . ')';
        }

        return DB::combine($wheres, $glue);
    }

    private function getChildIds($parent_ids = [], $level = 1)
    {
        if (empty($parent_ids))
        {
            return [];
        }

        $include_child_tags = $this->getAttribute('include-child-tags');

        if (is_numeric($include_child_tags) && $include_child_tags < $level)
        {
            return [];
        }

        $query = DB::getQuery()
            ->select('tag.id')
            ->from(DB::quoteName('#__tags', 'tag'))
            ->where(DB::is('tag.parent_id', $parent_ids, ['handle_wildcards' => false]));

        $children = DB::getResults($query);

        if (empty($children))
        {
            return [];
        }

        return array_merge($children, $this->getChildIds($children, $level++));
    }

    private function getIdsByValues($values, $glue = 'OR')
    {
        $query = DB::getQuery()
            ->select('tag.id')
            ->from(DB::quoteName('#__tags', 'tag'))
            ->where($this->getSubQueriesWhere($values, $glue));

        return DB::getResults($query);
    }

    private function getSubQueriesForWheres($values, $glue = 'OR')
    {
        if (empty($values))
        {
            return '';
        }

        $tag_ids = $this->getIdsByValues($values, $glue);

        if ( ! empty($this->getAttribute('include-child-tags')))
        {
            $child_ids = $this->getChildIds($tag_ids);
            $tag_ids   = array_merge($tag_ids, $child_ids);
        }

        return (string) DB::getQuery()
            ->select(DB::quoteName('map.content_item_id'))
            ->from(DB::quoteName('#__contentitem_tag_map', 'map'))
            ->join('LEFT', DB::quoteName('#__tags', 'tag')
                . ' ON ' . DB::quoteName('tag.id') . ' = ' . DB::quoteName('map.tag_id'))
            ->where(DB::is('map.type_alias', 'com_content.article'))
            ->where(DB::is('tag.id', $tag_ids));
    }

    private function getSubQueriesWhere($values, $glue = 'OR')
    {
        if (in_array('=+', $values))
        {
            return '('
                . DB::quoteName('tag.id') . ' != ' . DB::quote('')
                . ' AND '
                . DB::quoteName('tag.id') . ' IS NOT NULL'
                . ')';
        }

        return DB::is(['tag.id', 'tag.title', 'tag.alias'], $values, compact('glue'));
    }
}
